#pragma once

#include <iostream>
#include <allegro5/allegro5.h>
#include <allegro5/allegro_primitives.h>
#include "Globals.h"

/*! @brief Clase GameObject.
*			Clase base para el proyecto.
*
* Contiene todos los metodos y atributos necesarios para que un objeto interactue
* con"Player" y los dems.
*/
class GameObject{

private:
	int ID;/**< Entero que clasifica su identidad. vease enum ID en Globals.h */
	bool alive;/**< Booleano, true si el objeto an vive(player, enemy, boss, etc). */
	bool collidable;/**< Booleano, true si el objeto puede colisionar con otros. */

protected:
	int x;/**< Entero, posicin en eje x. */
	int y;/**< Entero, posicin en eje y. */

	int velX;/**< Entero, velocidad en eje x. */
	int velY;/**< Entero, velocidad en eje y. */

	int dirX;/**< Entero, direccin en eje x. */
	int dirY;/**< Entero, direccin en eje y. */

	int boundX;/**< Entero, N de pixeles desde el centro hacia la frontera en eje x. */
	int boundY;/**< Entero, N de pixeles desde el centro hacia  frontera en eje y. */

	int maxFrame;/**< Entero, N de frames (dibujos en sprite). */
	int curFrame;/**< Entero, contabiliza el nmero de frames de movimiento. */
	int frameWidth;/**< Entero, ancho del sprite. */
	int frameHeight;/**< Entero, alto del sprite. */
	int animationColumns;/**< Entero, columnas de sprites(En la imagen "image"). */
	int animationRow;/**< Entero que representa la fila del sprite(imagen). */

	ALLEGRO_BITMAP *image;/**< Puntero a un mapa de bits(imagenes con sprites). */

public:
	GameObject();
	void virtual Destroy();
	void Init(float x, float y, float velX, float velY, int dirX, int dirY, int boundX, int boundY);
	void virtual Update();
	void virtual Render()=0;

	int GetX() {return x;}
	int GetY() {return y;}

	void SetX(int x) {GameObject::x = x;}
	void SetY(int y) {GameObject::y = y;}

	int GetBoundX() {return boundX;}
	int GetBoundY() {return boundY;}

	int getDirX() { return dirX; }
	int getDirY() { return dirY; }

	int GetID() {return ID;}
	void SetID(int ID) {GameObject::ID = ID;}

	bool GetAlive() {return alive;}
	void SetAlive(bool alive) {GameObject::alive = alive;}

	bool GetCollidable() {return collidable;}
	void SetCollidable(bool collidable) {GameObject::collidable = collidable;}

	bool CheckCollisions(GameObject *otherObject);
	void virtual Collided(int objectID);
	bool Collidable();
};