#include "Player.h"

Player::Player(ALLEGRO_BITMAP *image) {
	Init(image);
}

void Player::Destroy(){
	GameObject::Destroy();
	al_destroy_sample(get);
}

void Player::Init(ALLEGRO_BITMAP *image){
	GameObject::Init(320, 240, 3, 3, 0, 0, 16, 16);
	gun = false;
	k1 = false;
	k2 = false;
	k3 = false;
	k4 = false;
	k5 = false;
	k6 = false;
	k7 = false;
	get = al_load_sample("sonidos/got.ogg");
	SetID(PLAYER);
	SetAlive(true);

	room = 0;
	change = false;

	life = 10;
	deads = 0;
	max_Life = life;
	score = 0;

	curFrame = 1;
	frameWidth = 32;
	frameHeight = 32;
	animationColumns = 3;
	animationRow = 0;

	if (image != NULL)
		Player::image = image;
}
/*! @brief Metodo new_Init.
*			Reinicia el estado del personaje.
*
* Al morir es necesario que el personaje vuelva a la vida, pero tambin que no pierda nada de lo que gan antes de morir,
* por esta razn, el mtodo new_Init nos permite recuperar nuestro avance y spawnear en una sala anterior fuera del alcanse
* del "Boss" o enemigo.
*/
void Player::new_Init(){
	SetAlive(true);
	life = max_Life;
	curFrame = 1;
	SetX(320);
	SetY(240);
	if (deads != 0) {
		change = true;
	}
}


void Player::Update(){
	GameObject::Update();
	if (x < 0)
		x = 0;
	else if (x > WIDTH)
		x = WIDTH;

	if (y < 0)
		y = 0;
	else if (y > HEIGHT)
		y = HEIGHT;
	for (iter = ammo.begin(); iter != ammo.end();) {
		(*iter)->Update();
		if (!(*iter)->GetAlive()) {
			(*iter)->Destroy();
			delete(*iter);
			iter = ammo.erase(iter);
		}
		else { iter++; }
	}
}
/*! @brief Metodo Render.
*			Dibuja, pinta.
*
* Dibuja el o los objetos.
*/
void Player::Render() {
	int fx = ((curFrame/3) % animationColumns) * frameWidth;
	int fy = animationRow * frameHeight;

	al_draw_bitmap_region(image, fx, fy, frameWidth, frameHeight, x - frameWidth / 2, y - frameHeight / 2, 0);
	for (iter = ammo.begin(); iter != ammo.end(); iter++)
		(*iter)->Render();
}
/*! @brief Metodo shoot.
*			Disparar el arma.
*
* Este mtodo inicia el sonido de disparo y crea un objeto "Bullet".
*/
void Player::shoot(){
	Bullet *bullet = new Bullet(this->GetX(), this->GetY(), this->GetPosition());
	ammo.push_back(bullet);
}
/*! @brief Metodo destroy_ammo.
*			Destruye las balas.
*
* Elimina todo objeto "Bullet" dentro de la lista "ammo".
*/
void Player::destroy_ammo(){
	for (iter = ammo.begin(); iter != ammo.end(); iter++)
		(*iter)->Destroy();
}
/*! @brief Metodo MoveUp.
*			Mover hacia arriba.
*
* A travs de eventos del teclado, el personaje se mueve hacia arriba.
*/
void Player::MoveUp(){
	animationRow = 3;
	dirY = -1;
	curFrame++;
}
/*! @brief Metodo MoveDown.
*			Mover hacia abajo.
*
* A travs de eventos del teclado, el personaje se mueve hacia abajo.
*/
void Player::MoveDown(){
	animationRow = 0;
	dirY = 1;
	curFrame++;
}
/*! @brief Metodo MoveLeft.
*			Mover hacia la izquierda.
*
* A travs de eventos del teclado, el personaje se mueve hacia la izquierda.
*/
void Player::MoveLeft(){
	animationRow = 1;
	curFrame++;
	dirX = -1;
}
/*! @brief Metodo MoveRight.
*			Mover hacia la derecha.
*
* A travs de eventos del teclado, el personaje se mueve hacia la derecha.
*/
void Player::MoveRight(){
	animationRow = 2;
	curFrame++;
	dirX = 1;
}
/*! @brief Metodo ResetAnimation.
*			Restaurar velocidades.
*
* Al detenerse en el ejex o ejey, su velocidad en ese eje se iguala a 0.
*/
void Player::ResetAnimation(int position){
	if (position == 1){
		dirY = 0;
	}
	else{
		dirX = 0;
	}
}

void Player::Collided(int objectID) {
	if (objectID == ENEMY)
		life--;
	if (objectID == BULLET)
		life -= 5;
	if (objectID == GUN && gun == false) {
		al_play_sample(get, 1, 0, 1, ALLEGRO_PLAYMODE_ONCE, 0);
		gun = true;
	}
	if (objectID == DOOR) {
		change = true;
	}
	if (objectID == KEYS) {
		al_play_sample(get, 1, 0, 1, ALLEGRO_PLAYMODE_ONCE, 0);
	}
}